==> externalVector_1.0.3.tar.gz: externalVector/R/storage.R <==
setClass("nativeStorageMethods",
         representation(ptr="externalptr"),
         contains = "externalResource")

setMethod("initializeResource",
          signature(resource="nativeStorageMethods"),
          function (resource, ptr, size, type, ...) 
      {
          resource@ptr <- ptr
          resource
      })

setMethod("allocator",
          signature(resource="nativeStorageMethods"),
          function (resource) new("gcAllocator"))

setMethod("getPointer",
          signature(resource="nativeStorageMethods"),
          function (resource) resource@ptr)

setMethod("allocatedSize",
          signature(resource="nativeStorageMethods"),
          function (resource)
          .Call("storageMethodsStructSize", PACKAGE="externalVector"))

setClass("nativeStorageMethodsList",
         representation(allMethods="nativeStorageMethods",
                        numericMethods="nativeStorageMethods",
                        integerMethods="nativeStorageMethods",
                        logicalMethods="nativeStorageMethods",
                        complexMethods="nativeStorageMethods",
                        characterMethods="nativeStorageMethods",
                        listMethods="nativeStorageMethods"))

setMethod("initialize",
          signature(.Object = "nativeStorageMethodsList"),
          function (.Object, ...) 
      {
          .Object@allMethods <- new("nativeStorageMethods")
          .Object@logicalMethods <- new("nativeStorageMethods")
          .Object@integerMethods <- new("nativeStorageMethods")
          .Object@numericMethods <- new("nativeStorageMethods")
          .Object@complexMethods <- new("nativeStorageMethods")
          .Object@characterMethods <- new("nativeStorageMethods")
          .Object@listMethods <- new("nativeStorageMethods")
          .Object
      })

setClass("externalStorage",
         representation(ptr="externalptr",
                        type="vector",
                        length="integer"),
         contains = c("VIRTUAL", "externalResource"),
         prototype = list(type = logical(1),
                          length = as.integer(0)))

setMethod("allocator",
          signature(resource="externalStorage"),
          function (resource) 
          new("gcAllocator"))

setMethod("initialize",
          signature(.Object="externalStorage"),
          function(.Object,
                   type = logical(1),
                   length = as.integer(0), ...)
      {
          if (!(class(type) %in% c("logical", "integer", "numeric",
                                   "complex", "character",
                                   "list")))
              stop("invalid storage type with class ", class(type))
          if (length(type) < 1)
              .Object@type <- vector(class(type), 1)
          else if (length(type) > 1)
              .Object@type <- type[1]
          else .Object@type <- type
          if (length < 1)
              .Object@length <- as.integer(0)
          else .Object@length <- as.integer(length)
              
          callNextMethod()
      })

setGeneric("getNativeStorageMethods",
           function(x) standardGeneric("getNativeStorageMethods"))

setExternalStorageClass <-
local({
    .nativeMethods <- new.env()

    assign("externalStorage",
           delay(.Call("initNativeStorageMethods",
                       new("nativeStorageMethodsList"),
                       PACKAGE="externalVector")),
           envir = .nativeMethods)

    setMethod("getNativeStorageMethods", "externalStorage",
              function(x)
          {
              Class <- class(x)
              if (exists(Class, envir=.nativeMethods, inherits=FALSE)) {
                  methods <- get(class(x), envir=.nativeMethods)
                  getPointer(slot(methods,
                                  paste(class(internalType(x)),
                                        "Methods", sep="")))
              } else {
                  stop("could not find storage methods for class \"",
                       Class, "\"")
              }
          })

    setNativeStorageMethods <-
        function(Class, methodNames, supClass, PACKAGE)
        {
            if (!isClass(Class))
                stop("Not a valid class:", Class)
            if (!extends(Class, "externalStorage")) {
                stop("Can not define native storage methods for objects not extending\nexternalStorage")
            }
            ## get the nativeStorageMethodsList object for the superclass
            supMethods <- get(supClass, envir=.nativeMethods)
            if (length(methodNames) > 0) {
                ## Create a list of external pointers - each containing the
                ## address of a native function
                thisMethodsList <- lapply(methodNames,
                                          function(mname)
                                      {
                                          getNativeSymbolInfo(mname,
                                                              PACKAGE)$address
                                      })
                names(thisMethodsList) <-
                    if (length(names(methodNames)) == 0)
                        methodNames
                    else ifelse(names(methodNames) == "",
                                methodNames, names(methodNames))
                .Call("createNativeStorageMethods",
                      new("nativeStorageMethodsList"), thisMethodsList,
                      supMethods, PACKAGE="externalVector")
            } else supMethods
        }

    ## setExternalStorageClass <-
    function(Class, methodNames, ...,
             contains = "externalStorage",
             where = topenv(parent.frame()))
    {
        supClasses <- contains[unlist(lapply(contains,
                                             extends,
                                             "externalStorage"))]
        if (length(supClasses) == 0)
            stop("invalid class definition: must contain externalStorage or its subclass")
        setClass(Class, ..., contains = contains, where = where)
        package <- attr(getClass(Class)@className, "package")
        assign(Class,
               delay(setNativeStorageMethods(Class, methodNames,
                                             supClasses, package),
                     environment()),
               envir=.nativeMethods)
        Class
    }
})

if (!isGeneric("internalType"))
    setGeneric("internalType",
               function(x)
               standardGeneric("internalType"))

setMethod("internalType",
          signature(x="externalStorage"),
          function(x)
      {
          type <- x@type
          if (class(type) %in% c("logical", "integer", "numeric",
                                 "complex", "character")) {
              type
          } else vector("list", 1)
      })

setMethod("initializeResource", "externalStorage",
          function(resource, ptr, size, type, ...)
       {
           resource@ptr <- ptr
           resource@type <- type
           resource@length <- size
           resource
       })

setMethod("getPointer",
          signature(resource="externalStorage"),
          function(resource)
      {
          resource@ptr
      })

==> externalVector_1.4.0.tar.gz: externalVector/R/storage.R <==
setClass("nativeStorageMethods",
         representation(ptr="externalptr"),
         contains = "externalResource")

setMethod("initializeResource",
          signature(resource="nativeStorageMethods"),
          function (resource, ptr, size, type, ...)
      {
          resource@ptr <- ptr
          resource
      })

setMethod("allocator",
          signature(resource="nativeStorageMethods"),
          function (resource) new("gcAllocator"))

setMethod("getPointer",
          signature(resource="nativeStorageMethods"),
          function (resource) resource@ptr)

setMethod("allocatedSize",
          signature(resource="nativeStorageMethods"),
          function (resource)
          .Call("storageMethodsStructSize", PACKAGE="externalVector"))

setClass("nativeStorageMethodsList",
         representation(allMethods="nativeStorageMethods",
                        numericMethods="nativeStorageMethods",
                        integerMethods="nativeStorageMethods",
                        logicalMethods="nativeStorageMethods",
                        complexMethods="nativeStorageMethods",
                        characterMethods="nativeStorageMethods",
                        listMethods="nativeStorageMethods"))

setMethod("initialize",
          signature(.Object = "nativeStorageMethodsList"),
          function (.Object, ...)
      {
          .Object@allMethods <- new("nativeStorageMethods")
          .Object@logicalMethods <- new("nativeStorageMethods")
          .Object@integerMethods <- new("nativeStorageMethods")
          .Object@numericMethods <- new("nativeStorageMethods")
          .Object@complexMethods <- new("nativeStorageMethods")
          .Object@characterMethods <- new("nativeStorageMethods")
          .Object@listMethods <- new("nativeStorageMethods")
          .Object
      })

setClass("externalStorage",
         representation(ptr="externalptr",
                        type="vector",
                        length="integer",
                        methodsptr="externalptr"),
         contains = c("VIRTUAL", "externalResource"),
         prototype = list(type = logical(1),
                          length = as.integer(0)))

setMethod("allocator",
          signature(resource="externalStorage"),
          function (resource)
          new("gcAllocator"))

setGeneric("getNativeStorageMethods",
           function(x) standardGeneric("getNativeStorageMethods"))

setMethod("getNativeStorageMethods",
          signature(x="ANY"),
          function (x) NULL)

setMethod("getNativeStorageMethods",
          signature(x="externalStorage"),
          function (x) x@methodsptr)

setExternalStorageClass <-
local({
    .nativeMethods <- new.env()

    delayedAssign("externalStorage",
           .Call("initNativeStorageMethods",
                       new("nativeStorageMethodsList"),
                       PACKAGE="externalVector"),
           assign.env = .nativeMethods)

#     setMethod("getNativeStorageMethods", "externalStorage",
#               function(x)
#           {
#               Class <- class(x)
#               if (exists(Class, envir=.nativeMethods, inherits=FALSE)) {
#                   methods <- get(class(x), envir=.nativeMethods)
#                   xstorage <- slot(methods,
#                                    paste(class(internalType(x)),
#                                          "Methods", sep=""))
#                   getPointer(xstorage)
#               } else {
#                   stop("could not find storage methods for class \"",
#                        Class, "\"")
#               }
#           })

    setMethod("initialize",
              signature(.Object="externalStorage"),
              function(.Object,
                       type = logical(1),
                       length = as.integer(0), ...)
          {
              if (!(class(type) %in% c("logical", "integer", "numeric",
                                       "complex", "character",
                                       "list")))
                  stop("invalid storage type with class ", class(type))
              if (length(type) < 1)
                  .Object@type <- vector(class(type), 1)
              else if (length(type) > 1)
                  .Object@type <- type[1]
              else .Object@type <- type
              if (length < 1)
                  .Object@length <- as.integer(0)
              else .Object@length <- as.integer(length)

              Class <- class(.Object)
              if (exists(Class, envir=.nativeMethods, inherits=FALSE)) {
                  methods <- get(Class, envir=.nativeMethods)
                  xstorage <- slot(methods,
                                   paste(class(internalType(.Object)),
                                         "Methods", sep=""))
                  .Object@methodsptr <- getPointer(xstorage)
              } else {
                  stop("could not find storage methods for class \"",
                       Class, "\"")
              }
              callNextMethod()
          })

    setNativeStorageMethods <-
        function(Class, methodNames, supClass, PACKAGE)
        {
            if (!isClass(Class))
                stop("Not a valid class:", Class)
            if (!extends(Class, "externalStorage")) {
                stop("Can not define native storage methods for objects not extending\nexternalStorage")
            }
            ## get the nativeStorageMethodsList object for the superclass
            supMethods <- get(supClass, envir=.nativeMethods)
            if (length(methodNames) > 0) {
                ## Create a list of external pointers - each containing the
                ## address of a native function
                thisMethodsList <- lapply(methodNames,
                                          function(mname)
                                      {
                                          getNativeSymbolInfo(mname,
                                                              PACKAGE)$address
                                      })
                names(thisMethodsList) <-
                    if (length(names(methodNames)) == 0)
                        methodNames
                    else ifelse(names(methodNames) == "",
                                methodNames, names(methodNames))
                .Call("createNativeStorageMethods",
                      new("nativeStorageMethodsList"), thisMethodsList,
                      supMethods, PACKAGE="externalVector")
            } else supMethods
        }

    ## setExternalStorageClass <-
    function(Class, methodNames, ...,
             contains = "externalStorage",
             where = topenv(parent.frame()))
    {
        supClasses <- contains[unlist(lapply(contains,
                                             extends,
                                             "externalStorage"))]
        if (length(supClasses) == 0)
            stop("invalid class definition: must contain externalStorage or its subclass")
        setClass(Class, ..., contains = contains, where = where)
        package <- attr(getClass(Class)@className, "package")
        delayedAssign(Class,
               setNativeStorageMethods(Class, methodNames,
                                             supClasses, package),
                     eval.env=environment(),
               assign.env=.nativeMethods)
        Class
    }
})

setGeneric("internalType",
               function(x)
               standardGeneric("internalType"))

setMethod("internalType",
          signature(x="externalStorage"),
          function(x)
      {
          type <- x@type
          if (class(type) %in% c("logical", "integer", "numeric",
                                 "complex", "character")) {
              type
          } else vector("list", 1)
      })

setMethod("initializeResource", "externalStorage",
          function(resource, ptr, size, type, ...)
       {
           resource@ptr <- ptr
           resource@type <- type
           resource@length <- size
           resource
       })

setMethod("getPointer",
          signature(resource="externalStorage"),
          function(resource)
      {
          resource@ptr
      })
